@@ -27,12 +27,10 @@ __all__ = [
 import os
 import sys
 import logging
+from ..log import logger
 import functools
 UNSLOTH_ENABLE_LOGGING  = os.environ.get("UNSLOTH_ENABLE_LOGGING",  "0") == "1"
 UNSLOTH_COMPILE_DISABLE = os.environ.get("UNSLOTH_COMPILE_DISABLE", "0") == "1"
-logger = logging.getLogger(__name__)
-if UNSLOTH_ENABLE_LOGGING:
-    logger.setLevel(logging.DEBUG)
 
 # Get only allowed options
 import inspect
@@ -110,7 +108,7 @@ def get_torch_compile_options(
         # "cuda.use_fast_math"                : True, # Disable fast math
         # Causes incompatible gradient sizes on 2.6
         # And TypeError: bad operand type for unary -: 'SymbolicCallArg'
-        "combo_kernels"                       : False,
+        "combo_kernels"                       : combo_kernels,
         "benchmark_combo_kernel"              : True,
         "combo_kernel_foreach_dynamic_shapes" : True,
     }
